<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon;

/**
 * Autoloads Classes
 *
 * This class include files if a object was created
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class Autoloader {

    /**
     * Sets the autoloader function
     */
    public static function register() {
        spl_autoload_register(array(__CLASS__, 'load')); // Autoloader angeworfen
    }

    /**
     * Loads a class
     *
     * @param   string      $class      Try to include this class
     */
    public static function load($class) {
        $file = trim($class, '\\');  // Evntl. vorangegangenen Backslashes entfernen
        $file_array = explode('\\', $file);
        array_shift($file_array); // Root-Namespace entfernen

        $file = implode(DIRECTORY_SEPARATOR, $file_array);
        $path = dirname(__DIR__) . DIRECTORY_SEPARATOR . strtolower($file . '.php');

        if (file_exists($path)) {
            /**
             * Includes the class
             */
            require_once $path; // Pfad der Klasse einbinden
        }
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "autoloader";
    }
}
