<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Core;

//use MAM\Daemon\Core\MAM\Daemon\Core\Utils as Utils;

/**
 * Utils Class
 *
 * This class probids access to the core fuctions and to libraries
 *
 * Helps plugin authors with basic functions
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
abstract class Utils {
    /**
     * Class for mam related stuff
     *
     * @var \MAM\Daemon\Core\Mod
     * @var \MAM\Daemon\Engine\Quake3\Mod
     * @var \MAM\Daemon\Engine\Frostbite3\Mod
     */
    protected $mod;
    /**
     * Class for Remote Commands
     *
     * @var \MAM\Daemon\Engine\Quake3\Rcon\RconCommands
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\RconCommands
     */
    protected $rcon;
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Player
     * @var \MAM\Daemon\Engine\Quake3\Player
     * @var \MAM\Daemon\Engine\Frostbite3\Player
     */
    protected $players;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;
    /**
     * Access MAM Database
     *
     * This is a predefined connection to MAM's default Database
     *
     * @var \MAM\Daemon\Libraries\Database\Database Database Wrapper
     */
    protected $database;
    /**
     * GeoIP Functions
     *
     * @var \MAM\Daemon\Libraries\GeoIP\GeoIP
     */
    protected $geoip;
    /**
     * Registry Class to access stuff globaly
     *
     * @var \MAM\Daemon\Core\Registry
     */
    protected $registry;
    /**
     * Permissions Class
     *
     * With this class you can check if a player has the correct permissions to do s.th
     *
     * @var \MAM\Daemon\Libraries\Permissions\Permissions
     */
    protected $permissions;
    /**
     * Handles Plugins
     *
     * @var \MAM\Daemon\Libraries\Plugins\Pluginloader
     */
    protected $pluginloader;

    /**
     * Constructor
     *
     * adds the core features for plugins
     */
    public function __construct() {
        $registry = \MAM\Daemon\Core\Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = &$registry->players;
        $this->rcon = $registry->rcon;
        $this->logging = $registry->logging;
        $this->permissions = $registry->permissions;
        $this->database = $registry->database;
        $this->geoip = $registry->geoip;
        $this->registry = $registry;
    }

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        $this->pluginloader = \MAM\Daemon\Core\Registry::getInstance()->pluginLoader;
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {

    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        $class = explode('\\', get_class($this));
        $class_name = strtolower(array_pop($class));
        return $class_name;
    }

    /**
     * Is called when function doesn't exist
     *
     * @param string $name Name of the function
     * @param array $arguments Function parameters
     */
    public function __call($name, $arguments) {
        trigger_error("Function \"$name\"  does not exist in " . $this, E_USER_WARNING);
    }
}
