<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at => www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at => Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http =>//www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Engine\Frostbite3\Bf4;

use \MAM\Daemon\Libraries\Helper\Helper;
use MAM\Daemon\Core\Registry;

/**
 * Weapon Class for Battlefield 4
 *
 * This class contains all weapons for Battlefield 4
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Weapons {
    private $weapons;
    private $extensions;
    private $gadgets;
    private $vehicles;
    private $special;
    private $logging;

    public function __construct() {
        $registry = Registry::getInstance();
        $this->logging = $registry->logging;

        $this->getWeapons();
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return 'Battlefield 4 Weapons';
    }

    /**
     * Hardcoded weapons
     */
    public function getWeapons() {
        $json = Helper::parseJson(__DIR__ . DIRECTORY_SEPARATOR . "data" . DIRECTORY_SEPARATOR . "weapons.json");
        $this->weapons = $json["weapons"];
        $this->extensions = $json["extensions"];
        $this->gadgets = $json["gadgets"];
        /**
         * @todo sort Vehicles
         */
        $this->vehicles = $json["vehicles"];

        $this->special = $json["special"];
    }

    /**
     * Returns the name, section and short name from $index
     * @param $index
     * @return array
     */
    public function getWeapon($index) {
        $index = (is_array($index)) ? $index[0] : $index;
        if($index == null){
            return array("section" => "special", "name" => "none", "short" => "none");
        }
        /* For BF4 Vehicles */
        $index_array = explode("/", $index);
        $index = end($index_array);

        if (trim($index) == "") {
            return array("section" => "special", "name" => "none", "short" => "none");
        }

        foreach ($this->weapons as $section => $values) {
            foreach ($values as $short => $long) {
                if (strtolower($short) == strtolower($index)) {
                    return array("section" => $section, "name" => $this->weapons[$section][$short], "short" => $short);
                }
            }
        }
        foreach ($this->special as $short => $long) {
            if (strtolower($short) == strtolower($index)) {
                return array("section" => "special", "name" => $this->special[$short], "short" => $short);
            }
        }
        foreach ($this->gadgets as $section => $values) {
            foreach ($values as $short => $long) {
                if (strtolower($short) == strtolower($index)) {
                    return array("section" => $section, "name" => $this->gadgets[$section][$short], "short" => $short);
                }
            }
        }

        foreach ($this->vehicles as $short => $long) {
            if (strtolower($short) == strtolower($index)) {
                return array("section" => "vehicles", "name" => $this->vehicles[$short], "short" => $short);
            }
            /*
              foreach($this->vehicles as $section => $short) {
              if(strtolower($short) == strtolower($index)) {
              return array("section" => $section, "name" => $this->vehicles[$section][$short, "short" => $short]);
              } */
        }
        $index_array = explode("_", $index);
        if (count($index_array) >= 2) {
            if ($index_array[count($index_array) - 1] == "v2") {
                $extension = $index_array[count($index_array) - 3] . "_" . $index_array[count($index_array) - 2] . "_" . $index_array[count($index_array) - 1];
            } else {
                $extension = $index_array[count($index_array) - 2] . "_" . $index_array[count($index_array) - 1];
            }
            foreach ($this->extensions as $section => $values) {
                foreach ($values as $short => $long) {
                    if (strtolower($short) == strtolower($extension)) {
                        return array("section" => $section, "name" => $this->extensions[$section][$short], "short" => $short);
                    }
                }
            }
        }
        $this->logging->error("Weapon with index: $index not found");
    }
}
