<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Engine\Frostbite3;

/**
 * Contains all main functions
 *
 * This class contains all main functions of the AdminMod
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Mod extends \MAM\Daemon\Core\Mod {

    /**
     * Delete all players first and then create all player objects and add them to $this->players[].
     * This function uses rcon command "status" to build the player list. See $this->rconPlayerList().
     * @param boolean $getmap Set to true, if you want to add the map to $this->lastmaps.
     */
    public function syncPlayerlist($getmap = false) {

        //clear all players first
        $this->players = array();

        $map = false;
        $online = $this->rcon->rconPlayerList($map);
        if (count($online) == 0) {
            $this->logging->info("Synced playerlist with 'status' (0 players online)");
            return;
        }
        foreach ($online as $value) {
            $this->players[$value["guid"]] = new Player($value["guid"], $value["pid"], $this->removeColor($value["name"]));
            $this->players[$value["guid"]]->setIP($value["ip"]);
            $this->players[$value["guid"]]->setPBGuid($value["pbguid"]);
        //    echo $value["name"] .'=>'. $value["guid"]."\n";
        }
        if ($getmap) {
            $this->lastmaps[] = $map;
        }
        $this->logging->info("Synced playerlist with 'status' (" . count($this->players) . " players online)");
    }

    /**
     * Updates the Gametype.
     */
    public function updateGametype() {
        $this->gametype = $this->rcon->getMaps()->getShortGamemodeFromLong($this->rcon->rconGetGametype());
        $this->logging->info("Updated gametype: " . $this->getLongGametype($this->gametype));
    }

    /**
     * Returns the long name of a gametype.
     * @param string $short Gametype abbrevation
     * @return string Long name of the gametype (or the given short one if it couldn't be found)
     */
    public function getLongGametype($short) {
        $short = strtolower($short);
        $gametypes = $this->rcon->getMaps()->getGamemodesArray()["translation"];
        if (!array_key_exists($short, $gametypes)) {
            return $short;
        }

        return $this->getLngString($gametypes[$short]);
    }

    /**
     * Returns the long team names.
     * @param int $index team index
     * @return string
     */
    public function getLongTeamName($index) {

        $teams = $this->rcon->rconGetTeamnames();
        if (!array_key_exists($index, $teams)) {
            return "Unknown";
        }
        return $teams[$index];
    }
}
