<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Engine\Frostbite3\Rcon;

use \MAM\Daemon\Libraries\Logging;

/**
 * BF4 Rcon Log Class
 *
 * This Class logs all Rcon Actions to a file
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class RconLog extends Rcon {
    /**
     * Logging Object
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    private $rcon_logfile;

    /**
     * Constructs the object
     * @param String $address Server Address
     * @param int $port Server Query Port
     */
    public function __construct($address, $port) {
        $this->rcon_logfile = new \MAM\Daemon\Libraries\Logging\LogHandler('daemon/', 'frostbite_rcon');

        parent::__construct($address, $port);
    }

    /**
     * Connect to the Server
     */
    public function connect() {
        parent::connect();

        $this->writeLog("=================");
        $this->writeLog("Connection opened");
    }

    /**
     * Sends Rcon Command and receives the answer
     * @param String $str The rcon command you want to send
     * @return String The answer of the server
     */
    public function rcon($str) {
        $this->writeLog(" Input: $str");
        $s = parent::rcon($str);

        $lines = explode("\n", $s);
        foreach ($lines as $value) {
            $value = trim($value);
            //Hide Line if Line is empty
            if ($value == "") {
                continue;
            }
            $this->writeLog("Output: $value");
        }

        return $s;
    }

    /**
     * Kills the rcon connection
     */
    public function quit() {
        parent::quit();
        if (is_resource($this->log)) {
            $this->writeLog("Connection Closed");
            $this->writeLog("=================");
            fclose($this->log);
        }
    }

    /**
     * Writes into the logfile
     *
     * @param String $line The Logline
     */
    private function writeLog($line) {
        $timestamp = date("[d.m.y H:i:s] ");
        $this->rcon_logfile->write($timestamp . $line);
    }

    /**
     * Destructs the object
     */
    public function __destruct() {
        fclose($this->log);
    }
}
