<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Quake3;

use MAM\Daemon\Core\Utils;

/**
 * Game Commands
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class Commands extends Utils {

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "daemon";
    }

    /**
     * Constructor
     *
     * Runs the plugin
     */
    public function __construct() {
        parent::__construct();
        $this->runPlugin();
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerCommand("map", '~^map [\w. -]+$~i', "map", $this);
        $this->mod->registerCommand("restart", '~^restart( (1|0))?$~i', "restart", $this);
        $this->mod->registerCommand("killserver", false, "killServer", $this);
        $this->mod->registerCommand("maprotate", false, "mapRotate", $this);
        $this->mod->registerCommand("gametype", '~^gametype \w+$~i', "gametype", $this);
        $this->mod->registerCommand("nextmap", false, "nextmap", $this);
        $this->mod->registerCommand("setpw", '~^setpw .+$~i', "setpw", $this);
        $this->mod->registerCommand("removepw", false, "removepw", $this);
        $this->mod->registerCommand("hardcore", '~^hardcore (1|0)$~i', "hardcore", $this);
        $this->mod->registerCommand("oldschool", '~^oldschool (1|0)$~i', "oldschool", $this);
        $this->mod->registerCommand("killcam", '~^killcam (1|0)$~i', "killcam", $this);
        $this->mod->registerCommand("gravity", '~^gravity( \d+)?$~i', "gravity", $this);
        $this->mod->registerCommand("knockback", '~^knockback( \d+)?$~i', "knockback", $this);
        $this->mod->registerCommand("speed", '~^speed( \d+)?$~i', "speed", $this);
        $this->mod->registerCommand("devmap", '~^devmap [\w ]+$~i', "devmap", $this);
        $this->mod->registerCommand("setff", '~^setff (0|1|2|3)$~i', "setff", $this);
        $this->mod->registerCommand("maps", false, "maps", $this);
        $this->mod->registerCommand("exec", '~^exec \S+$~i', "exec", $this);
        $this->mod->registerCommand("ff", false, "ff", $this);
        $this->mod->registerCommand("setnextmap", '~^setnextmap [\w. -]+$~i', "setnextmap", $this);
        $this->mod->registerCommand("spectator", '~^spectator (0|1|2)$~i', "spectator", $this);
        $this->mod->registerCommand("voting", '~^voting (1|0)$~', "voting", $this);
        $this->mod->registerCommand("scream", '~^scream .+$~i', "scream", $this);
        $this->mod->registerCommand("status", false, "status", $this);
        if ($this->registry->iceops) {
            $this->mod->registerCommand("screensay", '~^screensay .+$~i', "screensay", $this);
            $this->mod->registerCommand("screentell", '~^screentell .+$~i', "screentell", $this);
        }
    }

    /**
     * Change the current map
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters part of mapname
     */
    public function map($guid, $parameters) {
        //no case sensitivity
        $parameters = array_map("strtolower", $parameters);

        if (count($parameters) > 1 && in_array($parameters[count($parameters) - 1], array("sd", "war", "dm", "sab", "koth", "dom", "ctf", "htf", "twar", "tdm", "hq", "gtnw", "oneflag", "dd", "arena"))) {
            $gametype = array_pop($parameters);
        } else {
            $gametype = $this->mod->getCurrentGametype();
        }

        $map = $this->mod->findMap(implode(" ", $parameters));

        if (!$map) {
            $this->players[$guid]->say($this->mod->getLngString("mapNotFound", array("<MAP>"), array(implode(" ", $parameters))));
            return;
        }

        $this->rcon->rconSay($this->mod->getLngString("changeMap", array("<MAP>", "<GAMETYPE>"), array($this->mod->getLongMapName($map), $this->mod->getLongGametype($gametype))));
        if (!$this->rcon->rconChangeMap($map, $gametype)) {
            $this->rcon->rconSay($this->mod->getLngString("mapNotFound", array("<MAP>"), array(implode(" ", $parameters))));
        }
    }

    /**
     * Restart the current map
     *
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     * @return void
     */
    public function restart($guid, $parameters) {
        if (!empty($parameters[0])) {
            $this->rcon->rconSay($this->mod->getLngString("mapRestart"));
            $this->rcon->rconMapRestart(true);
        } else {
            $this->rcon->rconSay($this->mod->getLngString("mapFastRestart"));
            $this->rcon->rconMapRestart(false);
        }
    }

    /**
     * Kill the GameServer
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function killServer($guid, $parameters) {
        $this->rcon->rconKillServer();
    }

    /**
     * Loads the next map in the rotation
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function mapRotate($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getLngString("loadingNextMap"));
        $this->rcon->rconMapRotate();
    }

    /**
     * gametype changes the gametype reload the current map with the selected gametype directly
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function gametype($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getLngString("changeGametype", array("<GAMETYPE>"), array($this->mod->getLongGametype($parameters[0]))));
        $this->rcon->rconSetGametype($parameters[0]);
        $this->rcon->rconMapRestart(true);
    }

    /**
     * returns a message with the next map to the console
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function nextmap($guid, $parameters) {
        $next = $this->rcon->rconGetNextMap();
        $map = $this->mod->getLongMapName($next["map"]);
        $gt = $this->mod->getLongGametype($next["gametype"]);
        $this->rcon->rconSay($this->mod->getLngString("nextmap", array("<GAMETYPE>", "<MAP>"), array($gt, $map)));
    }

    /**
     * sets a password on the gameserver
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function setpw($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getLngString("passwordSet"));
        $this->rcon->rconSetDvar("g_password", implode(" ", $parameters));
    }

    /**
     * removes a password from the gameserver
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function removepw($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getLngString("passwordRemoved"));
        $this->rcon->rconSetDvar("g_password", "");
    }

    /**
     * turn hardcore on/off on the server for the next map/round
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function hardcore($guid, $parameters) {
        $mode = array($this->mod->getLngString("disabled"), $this->mod->getLngString("enabled"));
        $this->rcon->rconSay($this->mod->getLngString("hardcoreChange", array("<MODE>"), array($mode[$parameters[0]])));
        $this->rcon->rconSetDvar("scr_hardcore", $parameters[0]);
    }

    /**
     * enable or disabled ingame vote
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function voting($guid, $parameters) {
        $mode = array($this->mod->getLngString("disabled"), $this->mod->getLngString("enabled"));
        $this->rcon->rconSay($this->mod->getLngString("votingChange", array("<MODE>"), array($mode[$parameters[0]])));
        $this->rcon->rconSetDvar("ui_allowvote", $parameters[0]);
    }

    /**
     * turn oldschool on/off on the server for the next map/round
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function oldschool($guid, $parameters) {
        $mode = array($this->mod->getLngString("disabled"), $this->mod->getLngString("enabled"));
        $this->rcon->rconSay($this->mod->getLngString("oldschoolChange", array("<MODE>"), array($mode[$parameters[0]])));
        $this->rcon->rconSetDvar("scr_oldschool", $parameters[0]);
    }

    /**
     * turn killcam on/off on the server for the next map/round
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function killcam($guid, $parameters) {
        $mode = array($this->mod->getLngString("disabled"), $this->mod->getLngString("enabled"));
        $this->rcon->rconSay($this->mod->getLngString("killcamChange", array("<MODE>"), array($mode[$parameters[0]])));
        $this->rcon->rconSetDvar("scr_game_allowkillcam", $parameters[0]);
    }

    /**
     * sets the gravity to x on the server
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function gravity($guid, $parameters) {
        if (!empty($parameters[0])) {
            $value = $parameters[0];
        } else {
            $value = 800;
        }
        $this->rcon->rconSay($this->mod->getLngString("gravityChange", array("<VALUE>"), array($value)));
        $this->rcon->rconSetDvar("g_gravity", $value);
    }

    /**
     * sets the knockback to x on the server
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function knockback($guid, $parameters) {
        if (!empty($parameters[0])) {
            $value = $parameters[0];
        } else {
            $value = 1000;
        }
        $this->rcon->rconSay($this->mod->getLngString("knockbackChange", array("<VALUE>"), array($value)));
        $this->rcon->rconSetDvar("g_knockback", $value);
    }

    /**
     * sets the speed to x on the server
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function speed($guid, $parameters) {
        if (!empty($parameters[0])) {
            $value = $parameters[0];
        } else {
            $value = 190;
        }
        $this->rcon->rconSay($this->mod->getLngString("speedChange", array("<VALUE>"), array($value)));
        $this->rcon->rconSetDvar("g_speed", $value);
    }

    /**
     * start a map with enabled cheats
     *
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     * @return void
     */
    public function devmap($guid, $parameters) {
        //no case sensitivity
        $parameters = array_map("strtolower", $parameters);

        $map = $this->mod->findMap(implode(" ", $parameters));

        if (!$map) {
            $this->players[$guid]->say($this->mod->getLngString("mapNotFound", array("<MAP>"), array(implode(" ", $parameters))));
            return false;
        }

        $this->rcon->rconSay($this->mod->getLngString("changeMapDevMap", array("<MAP>"), array($this->mod->getLongMapName($map))));
        if (!$this->rcon->rconDevMap($map)) {
            $this->rcon->rconSay($this->mod->getLngString("mapNotFound", array("<MAP>"), array(implode(" ", $parameters))));
        }
    }

    /**
     * retuns a message that shows the friendly fire
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function ff($guid, $parameters) {
        $ff = $this->rcon->rconGetDvar("scr_team_fftype");
        $mode = array(
            $this->mod->getLngString("disabled"),
            $this->mod->getLngString("enabled"),
            $this->mod->getLngString("reflected"),
            $this->mod->getLngString("shared"),
        );
        $this->rcon->rconSay($this->mod->getLngString("friendlyFire", array("<MODE>"), array($mode[$ff])));
    }

    /**
     * sets the friendly fire to x
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function setff($guid, $parameters) {
        $mode = array(
            $this->mod->getLngString("disabled"),
            $this->mod->getLngString("enabled"),
            $this->mod->getLngString("reflected"),
            $this->mod->getLngString("shared"),
        );
        $this->rcon->rconSay($this->mod->getLngString("setFriendlyFire", array("<MODE>"), array($mode[$parameters[0]])));
        $this->rcon->rconSetDvar("scr_team_fftype", $parameters[0]);
    }

    /**
     * shows the usable maps
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function maps($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getMaps("; "));
    }

    /**
     * execute a config of the gameserver
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function exec($guid, $parameters) {
        if ($this->rcon->rconExec($parameters[0])) {
            $this->players[$guid]->say($this->mod->getLngString("execSuccess", array("<FILE>"), array($parameters[0])));
        } else {
            $this->players[$guid]->say($this->mod->getLngString("execError", array("<FILE>"), array($parameters[0])));
        }
    }

    /**
     * sets the next map to x
     *
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     * @return void
     */
    public function setnextmap($guid, $parameters) {
        $parameters = array_map("strtolower", $parameters);

        //war,dm,sab,koth,sd,dom,gtnw,oneflag,ctf,dd,arena

        if (count($parameters) > 1 && in_array($parameters[count($parameters) - 1], array("sd", "war", "dm", "sab", "koth", "dom", "ctf", "htf", "twar", "tdm", "hq", "gtnw", "oneflag", "dd", "arena"))) {
            $gametype = array_pop($parameters);
        } else {
            $gametype = $this->mod->getCurrentGametype();
        }

        $map = $this->mod->findMap(implode(" ", $parameters));

        if (!$map) {
            $this->players[$guid]->say($this->mod->getLngString("mapNotFound", array("<MAP>"), array(implode(" ", $parameters))));
            return false;
        }

        $this->rcon->rconSetNextMap($map, $gametype, true);

        $this->rcon->rconSay($this->mod->getLngString("setNextMap", array("<MAP>", "<GAMETYPE>"), array($this->mod->getLongMapName($map), $this->mod->getLongGametype($gametype))));
    }

    /**
     * Set Spectatormode
     * @param String $guid Guid of executing Player
     * @param String $parameters Additional parameters Parameter of this command
     */
    public function spectator($guid, $parameters) {
        $mode = array($this->mod->getLngString("disabled"), $this->mod->getLngString("enabled"));
        $this->rcon->rconSay($this->mod->getLngString("spectatorChange", array("<MODE>"), array($mode[$parameters[0]])));
        $this->rcon->rconSetDvar("scr_game_spectatetype", $parameters[0]);
    }

    /**
     * status about the currently running adminmod
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function status($guid, $parameters) {
        $parsed = $this->registry->parser->getParsedLogLines();

        $bytes = \MAM\Daemon\Libraries\Helper\Helper::size($this->registry->parser->getParsedBytes());
        $time = \MAM\Daemon\Libraries\Helper\Helper::makeUptime2(time() - $this->registry->startuptime);
        $ram = \MAM\Daemon\Libraries\Helper\Helper::size(memory_get_usage());

        $msg = $this->mod->getLngString("status", array("<PARSEDLINES>", "<PARSEDBYTES>", "<UPTIME>", "<MEMUSAGE>"), array($parsed, $bytes, $time, $ram));
        $this->rcon->rconSay($msg);
    }

    /**
     * shuffle a map from the maps.cfg and set it as map
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function shufflemap($guid, $parameters) {
        $maparray = explode(",", $this->mod->getMaps(","));
        $maprandom = $maparray[array_rand($maparray, 1)];
        $gametype = $this->mod->getCurrentGametype();

        $this->rcon->rconSay($this->mod->getLngString("changeMap", array("<MAP>", "<GAMETYPE>"), array($this->mod->getLongMapName($maprandom), $this->mod->getLongGametype($gametype))));
        $this->rcon->rconChangeMap($maprandom, $gametype);
    }

    /**
     * shuffle a map from the maps.cfg and set it as nextmap
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function shufflenextmap($guid, $parameters) {
        $maparray = explode(",", $this->mod->getMaps(","));
        $maprandom = $maparray[array_rand($maparray, 1)];
        $gametype = $this->mod->getCurrentGametype();

        $this->rcon->rconSay($this->mod->getLngString("setNextMap", array("<MAP>", "<GAMETYPE>"), array($this->mod->getLongMapName($maprandom), $this->mod->getLongGametype($gametype))));
        $this->rcon->rconSetNextMap($maprandom, $gametype, true);
    }

    /**
     * Screams a message 9 times on the server
     *
     * @param String $guid Guid of executing player
     * @param String $params Additional parameters
     */
    public function scream($guid, $params) {
        $msg = implode(" ", $params);
        $this->mod->findReason($msg);

        $msg = (strlen($msg) > 100) ? substr($msg, 0, 100) : $msg;
        for ($i = 0; $i <= 9; $i ++) {
            $this->rcon->rconSay("^$i$msg");
        }
    }

    /**
     * ScreenSay for CodX Versions
     *
     * Says a message in the middle of the screen
     *
     * @param String $guid Guid of executing player
     * @param String $params Additional parameters
     */
    public function screenSay($guid, $params) {
        $msg = implode(" ", $params);
        $this->mod->findReason($msg);
        $this->rcon->rcon('screensay ' . $msg);
    }

    /**
     * ScreenTell for CodX Versions
     *
     * Tells a message in the middle of the screen to a specific player
     *
     * @param String $guid Guid of executing player
     * @param String $params Additional parameters
     */
    public function screenTell($guid, $params) {
        $search = array_shift($params);

        $msg = implode(" ", $params);
        $this->mod->findReason($msg);

        $player = $this->mod->findPlayerGuid($search);
        if (!$player) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($parameters)));
            return false;
        }

        $this->rcon->rcon('screentell ' . $this->players[$player]->getPID() . ' ' . $msg);
    }
}
