<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Quake3;

/**
 * Contains all main functions
 *
 * This class contains all main functions of the adminmod
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Mod extends \MAM\Daemon\Core\Mod {
    /**
     * Teamnames
     *
     * @var array
     */
    protected $teamnames = array('allies' => '', 'axis' => '');

    /**
     * Returns the long weapon name.
     *
     * @param string $weapon
     * @return string
     */
    public function getLongWeaponName($weapon) {
        $weapons = &$this->weapons;

        if (!$weapon) {
            return $weapons["none"];
        }

        $image = "";
        if (is_array($weapon)) {
            list($weapon, $image) = $weapon;
        }

        if (array_key_exists($image, $weapons["mod"])) {
            return $weapons["mod"][$image];
        }

        if ($weapon == "none") {
            if (array_key_exists($image, $weapons["modnone"])) {
                return $weapons["modnone"][$image];
            }
            return $weapons["none"];
        }

        $parts = explode("_", $weapon);


        if ($parts[0] == "gl") {
            return $weapons["end"]["gl"] . " (" . $weapons["middle"][$parts[1]] . ")";
        }

        $start = "";
        $middle = "";
        $end = "";

        foreach ($parts as $value) {
            if (array_key_exists($value, $weapons["start"])) {
                $start = $weapons["start"][$value];
                continue;
            }
            if (array_key_exists($value, $weapons["middle"])) {
                $middle = $weapons["middle"][$value];
                continue;
            }
            if (array_key_exists($value, $weapons["end"])) {
                if ($end == "") {
                    $end = $weapons["end"][$value];
                } else {
                    $end = $end . " + " . $weapons["end"][$value];
                }
                continue;
            }
        }

        $string = $start . $middle;
        if (!empty($end)) {
            $string .= " ($end)";
        }

        return $string;
    }

    /**
     * Delete all players first and then create all player objects and add them to $this->players[].
     * This function uses rcon command "status" to build the player list. See $this->rconPlayerList().
     * @param boolean $getmap Set to true, if you want to add the map to $this->lastmaps.
     */
    public function syncPlayerlist($getmap = false) {
        //clear all players first
        $this->players = array();

        $map = false;
        foreach ($this->rcon->rconPlayerList($map) as $value) {
            $this->players[$value["guid"]] = new Player($value["guid"], $value["pid"], $this->removeColor($value["name"]));
        }
        if ($getmap) {
            $this->lastmaps[] = $map;
        }

        $this->logging->info("Synced playerlist with 'status' (" . count($this->players) . " players online)");
    }

    /**
     * Updates the Gametype Dvar (g_gametype).
     */
    public function updateGametype() {
        $this->gametype = $this->rcon->rconGetDvar("g_gametype");
        $this->logging->info("Updated Dvar g_gametype");
    }

    /**
     * Returns the long name of a gametype.
     * @param string $short Gametype abbrevation
     * @return string Long name of the gametype (or the given short one if it couldn't be found)
     */
    public function getLongGametype($short) {
        $short = strtolower($short);
        $gametypes = array("sd", "war", "dm", "sab", "koth", "dom", "ctf", "htf", "twar", "tdm", "hq");
        if (!in_array($short, $gametypes)) {
            return $short;
        }
        return $this->getLngString($short);
    }

    /**
     * Sets manuadminmod's Dvar VERSION.
     */
    public function setVersionDvar() {
        $this->rcon->rconSetsDvar("_manuadminmod", \MAM\Daemon\Core\Registry::getInstance()->version);
    }

    /**
     * Gets the long Bodypart Name.
     * @param string $bodypart
     * @return string response
     */
    public function getLongBodypart($bodypart) {
        $bodypart = strtolower($bodypart);
        $bodylist = array("torso_lower", "torso_upper", "right_arm_lower", "left_leg_upper", "neck", "right_arm_upper", "left_hand", "left_arm_lower", "none", "head", "right_leg_upper", "left_arm_upper", "right_leg_lower", "left_leg_lower", "left_foot", "right_foot", "right_hand");
        if (!in_array($bodypart, $bodylist)) {
            return $bodypart;
        }

        $bodypart = str_replace(array("_upper", "_lower"), array("", ""), $bodypart);

        return $this->getLngString($bodypart);
    }

    /**
     * Updates the names of the teams
     *
     * @return boolean true if teamnames were updated, false when not
     */
    public function updateTeamNames() {
        //Get team names
        $teamnames = $this->rcon->rconDvarList("scr_a*s");
        if (array_key_exists('scr_axis', $teamnames)) {
            $this->teamnames["axis"] = $teamnames["scr_axis"];
        } else {
            $this->teamnames["axis"] = 'axis';
        }

        if (array_key_exists('scr_allies', $teamnames)) {
            $this->teamnames["allies"] = $teamnames["scr_allies"];
        } else {
            $this->teamnames["allies"] = 'allies';
        }
        return true;
    }

    /**
     * Returns the long team names.
     *
     * @param string $short axis or allies
     * @return string Long name of the team
     */
    public function getLongTeamName($short) {

        if (!in_array($short, array("axis", "allies"), true)) {
            return "Unknown";
        }

        $teams = \MAM\Daemon\Libraries\Helper\Helper::parseJson('daemon' . DIRECTORY_SEPARATOR . 'engine' . DIRECTORY_SEPARATOR . $this->getEngine() . DIRECTORY_SEPARATOR . $this->getGame() . DIRECTORY_SEPARATOR . 'data' . DIRECTORY_SEPARATOR . 'teams.json');

        if (array_key_exists($this->teamnames[$short], $teams)) {
            return $teams[$this->teamnames[$short]];
        } else {
            return $this->teamnames[$short];
        }
    }
}
