#!/bin/bash
# Copyright www.ManuAdminMod.de
# License http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0

# check if screen is installed
command -v screen >/dev/null 2>&1 || (echo "install screen if you want to use this script"; exit 1)

# read mam.cfg and set vars
eval "$(cat mam.cfg | tr -d \\r)"

# own grep hook for exact matching
function screengrepname () {
	# \\. for the dot between pid and name
	# all dots in the var itself will be escaped
	# tab after the name
	screen -ls | grep "\\.${NAME//./\\.}	"
}

case $1 in

debug)
	if [[ $(screengrepname) ]]; then
		echo "$NAME is already started, use '$0 stop' to stop it!"
	else
		$PHP -f daemon.php -- -game "$GAME" -cfgdir "$CFGDIR" -logdir "$LOGDIR"
	fi
;;

restart)
	$0 stop
	$0 start
;;

show)
	tail -f "$LOGDIR/daemon/mod.log"
;;

start)
	if [[ $(screengrepname) ]]; then
		echo "$NAME is already started, use '$0 stop' or '$0 restart'"
	else
		screen -dmS "$NAME" /bin/bash -c "while true; do $PHP -f daemon.php -- -game \"$GAME\" -cfgdir \"$CFGDIR\" -logdir \"$LOGDIR\"; done"
		echo "$NAME was successfully started! If you have any problems, try '$0 debug' or read '$LOGDIR/daemon/mod.log'!"
	fi
;;

status)
	if [[ $(screengrepname) ]]; then
		echo "$NAME is currently STARTED!"
	else
		echo "$NAME is currently STOPPED!"
	fi
;;

stop)
	if [[ $(screengrepname) ]]; then
		kill $(screengrepname | awk -F . '{print $1}' | awk '{print $1}')
		echo "$NAME was successfully stopped!"
	else
		echo "$NAME is not running!"
	fi
;;

*)
	echo "Usage: $0 {debug|restart|show|start|status|stop}"
;;

esac
