<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\Punkbuster;

use MAM\Daemon\Core\Utils;

/**
 * Additional Commands for Servers with Punkbuster
 *
 * This Class adds additional commands for servers with pb
 * WIth this class you can take Screens of the players, execute a config or unbans a player
 *
 * @author ManuAdminMod
 * @todo Move all commands to 1 command with subcommands
 *
 */
class Punkbuster extends Utils {
    /**
     * True if pb is disabled
     *
     * @var boolean
     */
    private $disabled = false;

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->checkEnabled();

        $this->mod->registerCommand("pbupdate", false, "commandUpdate", $this);
        $this->mod->registerCommand("pbrestart", false, "commandRestart", $this);
        $this->mod->registerCommand("pbver", false, "commandVer", $this);
        $this->mod->registerCommand("pbbanlist", false, "commandBanList", $this);
        $this->mod->registerCommand("pbunban", "#^pbunban \d+$#i", "commandUnban", $this);
        $this->mod->registerCommand("pbclearbans", false, "commandClearbans", $this);
        $this->mod->registerCommand("pbbanguid", "#^pbbanguid [0-9a-z]+ ?(.*?)$#i", "commandBanGuid", $this);
        $this->mod->registerCommand("pbgetss", "#^pbgetss (.+?)$#i", "commandGetss", $this);
        $this->mod->registerCommand("pbload", "#^pbload(.*?)$#i", "commandLoad", $this);

        $this->mod->registerEvent("serverRestart", "eventServerRestart", $this);
    }

    /**
     * Executes the PB Command
     * @param string $cmd
     */
    private function execute($cmd) {
        if ($this->mod->getEngine() === "frostbite3") {
            $this->rcon->rcon(array("punkBuster.pb_sv_command", $cmd));
            return $this->registry->parser->rconLog()[1];
        } else {
            return $this->rcon->rcon($cmd);
        }
    }

    /**
     * Performs a check when the server was restarted
     */
    public function eventServerRestart() {
        $this->checkEnabled();
    }

    /**
     * Try to update pun
     */
    private function pbupdate() {
        $return = $this->execute("pb_sv_update");
        $this->logging->info("Punkbuster: Update command executed (pb_sv_update)");
    }

    /**
     * Returns the pb version
     *
     * @return array PB Version
     */
    private function pbver() {
        $return = $this->execute("pb_sv_ver");
        $pattern = '#\((v.*?) \| (A.*?) (C.*?)\)#i';
        preg_match($pattern, $return, $subpatterns);
        return array($subpatterns[1], $subpatterns[2], $subpatterns[3]);
    }

    /**
     * Restarts Punkbuster
     */
    private function pbrestart() {
        $return = $this->execute("pb_sv_restart");
        $this->logging->info("Punkbuster: Restart command executed (pb_sv_restart)");
    }

    /**
     * Clear the pb banlist
     */
    private function pbclearBans() {
        //pb_sv_BanEmpty

        $return = $this->execute("pb_sv_BanEmpty");
        $this->logging->info("Punkbuster: Banlist was cleared (pb_sv_BanEmpty)");
    }

    /**
     * Updates the pb perm banfile
     */
    private function pbupdateBanFile() {
        //pb_sv_UpdBanFile

        $return = $this->execute("pb_sv_UpdBanFile");
        $this->logging->info("Punkbuster: BanFile was updated (pb_sv_UpdBanFile)");
    }

    /**
     * Return parts of the banlist
     *
     * @param string $search The search pattern
     * @param NULL $entries
     * @param NULL $found
     * @return array
     */
    private function pbgetBanList($search = "", &$entries = NULL, &$found = NULL) {
        $return = $this->execute("pb_sv_banList \"" . $search . "\"");
        preg_match('#.*?: End of Ban List \((\d+) of (\d+) displayed\)#', $return, $subpatterns);
        $entries = $subpatterns[2];
        $found = $subpatterns[1];
        $return = substr($return, 0, 1e4);
        $lines = explode("\n", $return);
        $bans = array();
        //PunkBuster Server: 2   c90dc162f133749cec0cb632e5e01337 {10/1337} "FN! manu" "192.168.2.21:28961" haha HUHU
        $pattern = '#^.*?: (\d+)\s+([a-f0-9]+) \{(\d+)/(-?\d+)\} "(.*?)" "(.*?)"\s?(.*?)\s*$#i';
        foreach ($lines as $line) {
            if (preg_match($pattern, $line, $subpatterns)) {
                $bans[] = array(
                    "id" => $subpatterns[1],
                    "guid" => $subpatterns[2],
                    "minuteselapsed" => $subpatterns[3],
                    "banduration" => $subpatterns[4],
                    "name" => $subpatterns[5],
                    "ip" => $subpatterns[6],
                    "reason" => $subpatterns[7],
                );
            }
        }
        return $bans;
    }

    /**
     * Unbans a player
     *
     * @param int $id Banid
     * @return boolean true|false
     */
    private function pbunban($id) {
        $result = $this->execute("pb_sv_unban $id");
        $this->logging->info("Punkbuster: Deleting ban $id (pb_sv_unban)");
        return (stripos($result, "has been Unbanned") !== false) ? true : false;
    }

    /**
     * Bans a Guid
     *
     * @param string $guid Plöayer Guid
     * @param string $name Player Name
     * @param string $ip Player IP
     * @param string $reason Reason
     * @return boolean true|false
     */
    private function pbbanGuid($guid, $name = "???", $ip = "???", $reason = "") {
        $return = $this->execute("pb_sv_banguid $guid \"$name\" \"$ip\" \"$reason\"");
        return (stripos($return, "Ban Added to Ban List") !== false) ? true : false;
    }

    /**
     * Takes a screenshot of a player
     *
     * @param string $search search params
     * @return string
     */
    private function pbgetSS($search) {
        $result = $this->execute("pb_sv_getss $search");
        $pattern = '#.*?: (\d+) Screenshot Requested#i';
        preg_match($pattern, $result, $subpatterns);
        return $subpatterns[1];
    }

    /**
     * Loads a pb cfg
     *
     * @param string $file Filename
     * @return string|boolean
     */
    private function pbload($file = "") {
        $result = $this->execute("pb_sv_load $file");
        $pattern = '#.*?: file not found: (.*?)\s*#';
        if (empty($result)) {
            return true;
        } elseif (preg_match($pattern, $result, $subpatterns)) {
            return "File not found";
        } else {
            return "Unknown error";
        }
    }

    /**
     * The Command for Updates
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function commandUpdate($guid, $parameters) {
        if ($this->disabled) {
            return false;
        }
        $this->pbupdate();
        $this->players[$guid]->say($this->mod->getLngString("punkbuster_updatestarted"));
    }

    /**
     * Command for pb restart
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function commandRestart($guid, $parameters) {
        if ($this->disabled) {
            return false;
        }
        $this->pbrestart();
        $this->players[$guid]->say($this->mod->getLngString("punkbuster_restart"));
    }

    /**
     * Command for PB Version
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function commandVer($guid, $parameters) {
        if ($this->disabled) {
            return false;
        }
        $version = $this->pbver();
        $this->players[$guid]->say($this->mod->getLngString("punkbuster_version", array("{0}", "{1}", "{2}"), $version));
    }

    /**
     * Command for banlist listing
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function commandBanList($guid, $parameters) {
        if ($this->disabled) {
            return false;
        }
        $search = implode(" ", $parameters);
        $bans = $this->pbgetBanList($search, $entries, $found);
        if ($found > 5) {
            $this->players[$guid]->say($this->mod->getLngString("punkbuster_toomuchresults"));
        } else {
            $this->players[$guid]->say($this->mod->getLngString("punkbuster_bansfound", array("<FOUND>", "<TOTAL>"), array($found, $entries)));
            foreach ($bans as $ban) {
                $search = array("<ID>", "<GUID>", "<NAME>", "<REASON>", "<LEFT>");
                $replace = array(
                    str_pad($ban["id"], 4, " ", STR_PAD_LEFT),
                    substr($ban["guid"], 0, 6),
                    $ban["name"],
                    $ban["reason"],
                    (($ban["banduration"] > -1) ? (($ban["banduration"] - $ban["minuteselapsed"]) . "min") : $this->mod->getLngString("punkbuster_unlimited")),
                );
                $str = $this->mod->getLngString("punkbuster_ban", $search, $replace);
                $this->players[$guid]->say($str);
            }
        }
    }

    /**
     * Command for unban
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function commandUnban($guid, $parameters) {
        if ($this->disabled) {
            return false;
        }
        if ($this->pbunban($parameters[0])) {
            $this->players[$guid]->say($this->mod->getLngString("punkbuster_unbansuccess"));
            $this->pbupdateBanFile();
        } else {
            $this->players[$guid]->say($this->mod->getLngString("punkbuster_unbanfailed"));
        }
    }

    /**
     * Command for clearing the banlist
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function commandClearbans($guid, $parameters) {
        if ($this->disabled) {
            return false;
        }
        $this->pbclearBans();
        $this->players[$guid]->say($this->mod->getLngString("punkbuster_banscleared"));
        $this->pbupdateBanFile();
    }

    /**
     * Command for Guid Banning
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function commandBanGuid($guid, $parameters) {
        if ($this->disabled) {
            return false;
        }

        $banguid = $parameters[0];
        $name = (isset($parameters[1])) ? $parameters[1] : "???";
        $ip = (isset($parameters[2])) ? $parameters[2] : "???";
        $reason = (isset($parameters[3])) ? $parameters[3] : "";
        if ($this->pbbanGuid($banguid, $name, $ip, $reason)) {
            $this->players[$guid]->say($this->mod->getLngString("punkbuster_banguidsuccess"));
            $this->pbupdateBanFile();
        } else {
            $this->players[$guid]->say($this->mod->getLngString("punkbuster_banguidfailed"));
        }
    }

    /**
     * Command for screenshots
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandGetss($guid, $parameters) {
        $player = implode($parameters, " ");

        $toss = $this->mod->findPlayerGuid($player);

        if (!$toss) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($player)));
        } else {
            $requested = $this->pbgetSS($this->players[$toss]->getPbid());
            $this->players[$guid]->say($this->mod->getLngString("punkbuster_ssrequested", array("<REQUESTED>", "<PLAYER_NAME>"), array($requested, $this->players[$toss]->getName())));
        }
    }

    /**
     * Command for executing pb cfg
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandLoad($guid, $parameters) {
        $file = implode($parameters);
        $result = $this->pbload($file);
        if ($result === true) {
            $this->players[$guid]->say($this->mod->getLngString("punkbuster_loadsuccess"));
        } else {
            $this->players[$guid]->say($this->mod->getLngString("punkbuster_loaderror", "<ERROR>", $result));
        }
    }

    /**
     * Check if PB is enabled
     */
    private function checkEnabled() {
        $enabled = ($this->mod->getEngine() === "frostbite3") ? $this->rcon->rcon(array("punkBuster.isActive"))[1] : $this->rcon->rconGetDvar("sv_punkbuster");
        if (!$enabled) {
            $this->logging->notice("Punkbuster: PB is disabled on this server (sv_punkbuster = 0), please enable it before using this plugin");
            $this->disabled = true;
        } else {
            $this->disabled = false;
        }
    }
}
