<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\RandomMapCycle;

use MAM\Daemon\Core\Utils;

/**
 * Random Map Cycles
 *
 * Lets you enable a random map rotation
 *
 * @author manu (modified by belstgut for new ManuAdminMod version)
 */
class RandomMapCycle extends Utils {

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->mod->setDefaultCV('randommapcycle', 'enabled', false);
        $this->mod->setDefaultCV('randommapcycle', 'maps', $this->mod->getMaps());
        $this->mod->setDefaultCV('randommapcycle', 'gametypes', array($this->mod->getCurrentGametype()));
        $this->mod->setDefaultCV('randommapcycle', 'notlastmaps', 3);
        $this->mod->setDefaultCV('randommapcycle', 'notlastgt', true);
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerEvent('nextMap', 'randommapcycleSet', $this);
    }

    /**
     * Sets the random Mapcycle
     *
     * @return boolean
     */
    public function randommapcycleSet() {
        if (!$this->mod->getCV('randommapcycle', 'enabled')) {
            return false;
        }

        if ($this->mod->getCV('randommapcycle', 'notlastmaps') < 1) {
            $this->logging->warning('Randommapcycle: ConfigVar [randommapcycle]notlastmaps must be greater than 0, randommapcycle will be disabled');
            return false;
        }

        $lastmaps = $this->mod->getLastMaps(2);
        if ($this->mod->getCV('randommapcycle', 'maps') == '<all>') {
            $maps = $this->mod->getMaps();
        } else {
            $maps = $this->mod->getCV('randommapcycle', 'maps');
        }
        if ($this->mod->getCV('randommapcycle', 'gametypes') == '<all>') {
            $gametypes = $this->mod->getGametypes();
        } else {
            $gametypes = $this->mod->getCV('randommapcycle', 'gametypes');
        }

        if ($lastmaps[1] != $lastmaps[0]) {
            $randommapcycle_maps[] = $lastmaps[0];
            while (count($randommapcycle_maps) > $this->mod->getCV('randommapcycle', 'notlastmaps')) {
                array_shift($randommapcycle_maps);
            }

            foreach ($maps as $key => $value) {
                if (in_array($value, $randommapcycle_maps)) {
                    unset($maps[$key]);
                }
            }

            if (empty($maps)) {
                $this->logging->warning('Randommapcycle: Can\'t choose a random map because count(maps) = 0');
                return false;
            }

            $maps = array_merge($maps);

            if ($this->mod->getCV('randommapcycle', 'notlastgt')) {
                foreach ($gametypes as $key => $value) {
                    if ($value == $this->mod->getCurrentGametype()) {
                        unset($gametypes[$key]);
                    }
                }

                $gametypes = array_merge($gametypes);
            }

            if (empty($gametypes)) {
                //$this->logging->write(MOD_WARNING, 'Randommapcycle: Can't choose a gametype because count(\$gametypes) = 0, choosing current: '.$this->mod->getCurrentGametype());
                $gametypes[] = $this->mod->getCurrentGametype();
            }

            $randommap = $maps[rand(0, count($maps) - 1)];
            $randomgt = $gametypes[rand(0, count($gametypes) - 1)];

            $this->logging->notice('Randommapcycle: Setting nextmap to ' . $randommap . '(' . $randomgt . ')');
            $this->rcon->rconSetNextMap($randommap, $randomgt, false);
        }

        return true;
    }
}
