<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\Voting;

/**
 * Abstract Class for Vote
 *
 * @author ManuAdminMod
 */
abstract class Vote {
    /**
     * Class for mam related stuff
     *
     * @var \MAM\Daemon\Engine\Quake3\Mod
     * @var \MAM\Daemon\Engine\Frostbite3\Mod
     * @var \MAM\Daemon\Core\Mod
     */
    protected $mod;
    /**
     * Class for Remote Commands
     *
     * @var \MAM\Daemon\Engine\Quake3\Rcon\RconCommands
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\RconCommands
     */
    protected $rcon;
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Player
     * @var \MAM\Daemon\Engine\Frostbite3\Player
     * @var \MAM\Daemon\Engine\Quake3\Player
     *
     */
    protected $players;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;
    /**
     * Parameters for the vote
     *
     * @var string
     */
    protected $parameters;
    /**
     * Parent
     *
     * @var string
     */
    protected $parent;
    /**
     * Count of player who votes yes
     *
     * @var int
     */
    protected $yes = 0;
    /**
     * Count of players who votes no
     *
     * @var int
     */
    protected $no = 0;
    /**
     * Started
     *
     * @var boolean
     */
    protected $started = false;
    /**
     * Voted
     *
     * @var array
     */
    protected $voted = array();
    /**
     * Last ancounce in seconds
     *
     * @var int
     */
    protected $lastannounce;
    /**
     * Selfcanceled
     *
     * @var int
     */
    public $selfcanceled = false;

    /**
     * Constructor
     * @param String $parent
     * @param String $starter
     * @param String $parameters
     */
    public function __construct($parent, $starter, $parameters) {
        $registry = \MAM\Daemon\Core\Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = &$registry->players;
        $this->rcon = $registry->rcon;
        $this->logging = $registry->logging;

        $this->parameters = $parameters;
        $this->parent = $parent;
        $this->starter = $starter;
        $this->yes ++;
        $this->voted[] = $starter;
        $this->started = time();
        $this->lastannounce = time();

        $this->parameterCheck();

        $this->start();
    }

    /**
     * parameter Check
     */
    protected function parameterCheck() {

    }

    /**
     * When Player quit
     *
     * @param string $guid Player Guid
     */
    public function playerQuit($guid) {

    }
    //abstract public function getName();

    /**
     * Gets the start of a vote
     * @return string guid
     */
    public function getStarter() {
        return $this->starter;
    }
    //abstract static public function isAllowed();

    /**
     * Start a vote
     */
    abstract public function start();

    /**
     * Cancel a vote
     */
    abstract public function cancel();

    /**
     * Pass a vote
     */
    abstract public function pass();

    /**
     * Vote failed
     */
    abstract public function failed();

    /**
     * Announce a vote
     */
    abstract public function announce();

    /**
     * Player Voted yes
     *
     * @param string $player Guid of player
     * @return void
     */
    public function voteYes($player) {
        if (in_array($player, $this->voted)) {
            return;
        }
        $this->yes++;
        $this->voted[] = $player;
        $this->players[$player]->say($this->mod->getLngString("votedYes"));
    }

    /**
     * Player Voted no
     *
     * @param string $player Guid of player
     * @return void
     */
    public function voteNo($player) {
        if (in_array($player, $this->voted)) {
            return;
        }
        $this->no++;
        $this->voted[] = $player;
        $this->players[$player]->say($this->mod->getLngString("votedNo"));
    }

    /**
     * Returns the yes vote ratio
     * @return string
     */
    public function getVoteRatio() {
        return ($this->yes) / ($this->yes + $this->no);
    }

    /**
     * Get count of yes votes
     * @return int
     */
    public function getYes() {
        return $this->yes;
    }

    /**
     * Get count of no votes
     *
     * @return int
     */
    public function getNo() {
        return $this->no;
    }

    /**
     * Get sum of yes + no votes
     *
     * @return int
     */
    public function getVotersNum() {
        return ($this->yes + $this->no);
    }

    /**
     * Vote time is over
     */
    public function end() {
        if ($this->getVoteRatio() > 0.5) {
            $this->pass();
        } else {
            $this->failed();
        }
    }

    /**
     * Get vote parameters
     *
     * @return array
     */
    public function getParameters() {
        return $this->parameters;
    }

    /**
     * Trigger for everytime
     *
     * @return void
     */
    public function triggerEveryTime() {
        if ($this->started == 0) {
            return;
        }
        if (time() - $this->lastannounce > $this->mod->getCV("voting", "annouce")) {
            $this->announce();
            $this->lastannounce = time();
        }
    }

    /**
     * Gets the starter
     *
     * @return String
     */
    public function getStarted() {
        return $this->started;
    }
}
