<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Plugins\Voting;

/**
 * Vote for setting next map in maprotation
 *
 * @author ManuAdminMod
 */
class VoteSetnextmap extends \MAM\Plugins\Voting\Vote {

    /**
     * Checks the Parameters
     *
     * @throws \MAM\Plugins\Voting\VoteException
     */
    protected function parameterCheck() {
        if (empty($this->parameters[0])) {
            throw new \MAM\Plugins\Voting\VoteException($this->mod->getLngString("voteSpecifyMap"));
        }
        $search = implode(" ", $this->parameters);
        $map = $this->mod->findMap($search);
        if (!$map) {
            throw new \MAM\Plugins\Voting\VoteException($this->mod->getLngString("mapNotFound", array("<MAP>"), array($search)));
        }
        if ($this->mod->getCV("voting", "allowedmaps") == "<maps.json>" && !in_array($map, $this->mod->getMaps())) {
            throw new \MAM\Plugins\Voting\VoteException($this->mod->getLngString("voteMapNotAllowed"));
        }
        if ($this->mod->getCV("voting", "allowedmaps") != "<all>" && $this->mod->getCV("voting", "allowedmaps") != "<maps.json>" && !in_array($map, $this->mod->getCV("voting", "allowedmaps"))) {
            throw new \MAM\Plugins\Voting\VoteException($this->mod->getLngString("voteMapNotAllowed"));
        }
        $this->parameters = $map;
    }

    /**
     * Start a Vote
     */
    public function start() {
        $say = $this->mod->getLngString("voteStarted") . " " . $this->mod->getLngString("voteSetNextMap", "<MAP>", $this->mod->getLongMapName($this->parameters)) . " " . $this->mod->getLngString("voteYesNo");
        $this->rcon->rconSay($say);
    }

    /**
     * Cancel a vote
     */
    public function cancel() {
        $say = $this->mod->getLngString("voteCanceled") . " " . $this->mod->getLngString("voteSetNextMap", "<MAP>", $this->mod->getLongMapName($this->parameters));
        $this->rcon->rconSay($say);
    }

    /**
     * Pass a vote
     */
    public function pass() {
        $this->rcon->rconSay($this->mod->getLngString("votePassed") . " " . $this->mod->getLngString("voteSetNextMap", "<MAP>", $this->mod->getLongMapName($this->parameters)) . " " . $this->mod->getLngString("voteYesNo3", array("<YES>", "<NO>"), array($this->yes, $this->no)));

        $this->rcon->rconSetNextMap($this->parameters, $this->mod->getCurrentGametype(), true);
    }

    /**
     * Vote failed
     */
    public function failed() {
        $say = $this->mod->getLngString("voteFailed") . " " . $this->mod->getLngString("voteSetNextMap", "<MAP>", $this->mod->getLongMapName($this->parameters)) . " " . $this->mod->getLngString("voteYesNo3", array("<YES>", "<NO>"), array($this->yes, $this->no));
        $this->rcon->rconSay($say);
    }

    /**
     * Announce a vote
     */
    public function announce() {
        $say = $this->mod->getLngString("voteInProgress") . " " . $this->mod->getLngString("voteSetNextMap", "<MAP>", $this->mod->getLongMapName($this->parameters)) . " " . $this->mod->getLngString("voteYesNo2", array("<YES>", "<NO>"), array($this->yes, $this->no));
        $this->rcon->rconSay($say);
    }
}
