<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Plugins\Voting;

use MAM\Daemon\Core\Utils;

/**
 * Plugin Voting
 *
 * This plugin implements a complete voting system
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 * @todo Move comands to 1 command
 */
class Voting extends Utils {
    /**
     * Votes
     *
     * @var array
     */
    private $votes = array();
    /**
     * Parameters of the vote
     *
     * @var string
     */
    private $vote;
    /**
     * Map start
     *
     * @var int
     */
    private $mapstart = 0;
    /**
     * Vote started
     *
     * @var int
     */
    private $votestarted = 0;

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct();

        foreach (array_keys($this->players) as $guid) {
            $this->playerInit($guid);
        }

        $this->mapstart = time();
    }

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->mod->setDefaultCV("voting", "enabled", 0);
        $this->mod->setDefaultCV("voting", "duration", 45);
        $this->mod->setDefaultCV("voting", "interval", 120);
        $this->mod->setDefaultCV("voting", "annouce", 15);
        $this->mod->setDefaultCV("voting", "minplayers", 3);
        $this->mod->setDefaultCV("voting", "maxvotesperplayer", 2);
        $this->mod->setDefaultCV("voting", "allowkick", 0);
        $this->mod->setDefaultCV("voting", "allowban", 0);
        $this->mod->setDefaultCV("voting", "allowmap", 0);
        $this->mod->setDefaultCV("voting", "allowgametype", 0);
        $this->mod->setDefaultCV("voting", "allownextmap", 0);
        $this->mod->setDefaultCV("voting", "allowrestart", 0);
        $this->mod->setDefaultCV("voting", "allowsetnextmap", 0);
        $this->mod->setDefaultCV("voting", "allowhardcore", 1);
        $this->mod->setDefaultCV("voting", "allowedmaps", "<maps.json>");
        $this->mod->setDefaultCV("voting", "allowedgametypes", array("war", "dom", "koth", "dm", "sab", "sd"));
        $this->mod->setDefaultCV("voting", "banduration", 0);
        $this->mod->setDefaultCV("voting", "ignoremapchange", 0);
        $this->mod->setDefaultCV("voting", "timelimit", 0);
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();


        $this->registerVote("kick", "VoteKick");
        $this->registerVote("ban", "VoteBan");
        $this->registerVote("map", "VoteMap");
        $this->registerVote("nextmap", "VoteNextmap");
        $this->registerVote("restart", "VoteRestart");
        $this->registerVote("gametype", "VoteGametype");
        $this->registerVote("setnextmap", "VoteSetnextmap");
        $this->registerVote("hardcore", "VoteHardcore");

        $this->mod->registerCommand("vote", false, "commandVote", $this);
        $this->mod->registerCommand("avote", false, "commandAvote", $this);
        $this->mod->registerCommand("yes", false, "commandYes", $this);
        $this->mod->registerCommand("no", false, "commandNo", $this);
        $this->mod->registerCommand("cancel", false, "commandCancel", $this);
        $this->mod->registerCommand("pass", false, "commandPass", $this);

        $this->mod->registerEvent("nextMap", "triggerNextMap", $this);
        $this->mod->registerEvent("everyTime", "triggerEveryTime", $this);
        $this->mod->registerEvent("playerJoined", "playerInit", $this);
        $this->mod->registerEvent("playerSay", "playerSay", $this);
        $this->mod->registerEvent("playerQuit", "playerQuit", $this);
    }

    /**
     * Registers a new vote
     *
     * @param string $name Shortname for the vote
     * @param name $class Class related for the vote
     * @throws \Exception
     */
    public function registerVote($name, $class) {
        $var = __NAMESPACE__ . "\\" . $class;

        if (!class_exists($var)) {
            throw new \Exception("Class $class does not exist");
        }

        $this->votes[$name] = $class;
    }

    /**
     * Implements the vote command
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandVote($guid, $parameters) {
        if (!$this->mod->getCV("voting", "enabled")) {
            return;
        }
        if ($this->isVoteInProgress()) {
            return;
        }

        $parameters = array_map("strtolower", $parameters);


        if (empty($parameters[0]) || !array_key_exists($parameters[0], $this->votes)) {
            $this->players[$guid]->say($this->mod->getLngString("allowedVotes", "<ALLOWED>", implode(", ", $this->getAllowed())));
            return;
        }

        if (!$this->mod->getCV("voting", "allow" . $parameters[0])) {
            $this->players[$guid]->say($this->mod->getLngString("allowedVotes", "<ALLOWED>", implode(", ", $this->getAllowed())));
            return;
        }

        if ($this->players[$guid]->voting_votesstarted >= $this->mod->getCV("voting", "maxvotesperplayer")) {
            $this->players[$guid]->say($this->mod->getLngString("voteMaximumVotesReached"));
            return;
        }

        if (count($this->players) < $this->mod->getCV("voting", "minplayers")) {
            $this->players[$guid]->say($this->mod->getLngString("voteNotNotEnoughPlayers"));
            return;
        }

        if (time() - $this->votestarted < $this->mod->getCV("voting", "interval")) {
            $wait = $this->mod->getCV("voting", "interval") - (time() - $this->votestarted);
            $this->players[$guid]->say($this->mod->getLngString("votePleaseWait", "<WAIT>", $wait));
            return;
        }

        if ($this->mod->getCV("voting", "timelimit") != 0 && time() - $this->mapstart > $this->mod->getCV("voting", "timelimit") * 60) {
            $this->players[$guid]->say($this->mod->getLngString("voteTooLate"));
            return;
        }

        $vote = array_shift($parameters);

        try {
            $var = __NAMESPACE__ . '\\' . $this->votes[$vote];
            $this->vote = new $var($this, $guid, $parameters);
            $this->votestarted = time();
            $this->logging->info("Voting: Vote started: $vote (Parameters: " . implode(", ", $parameters) . ")");
        } catch (\MAM\Plugins\Voting\VoteException $e) {
            $this->players[$guid]->say($e->getMessage());
        }
    }

    /**
     * Returns all allowed votes
     *
     * @return array allowed votes
     */
    private function getAllowed() {
        $allowed = array();
        foreach ($this->votes as $key => $value) {
            if ($this->mod->getCV("voting", "allow" . $key)) {
                $allowed[] = $key;
            }
        }
        return $allowed;
    }

    /**
     * Inits the player
     *
     * @param String $guid Player Guid
     */
    public function playerInit($guid) {
        $this->players[$guid]->voting_votesstarted = 0;
    }

    /**
     * Command for !yes
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandYes($guid, $parameters) {
        if (!$this->isVoteInProgress()) {
            return;
        }

        $this->vote->voteYes($guid);
        $this->checkForEarlyEnd();
    }

    /**
     * Command for !no
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function commandNo($guid, $parameters) {
        if (!$this->isVoteInProgress()) {
            return;
        }

        $this->vote->voteNo($guid);
        $this->checkForEarlyEnd();
    }

    /** Check if more then 50% of the players voted */
    private function checkForEarlyEnd() {
        if ($this->vote->getYes() > count($this->players) / 2 || $this->vote->getNo() > count($this->players) / 2) {
            $this->vote->end();
            $this->logging->info("Voting: Vote ended (Yes: " . $this->vote->getYes() . ", No: " . $this->vote->getNo() . ")");
            $this->vote = null;
        }
    }

    /**
     * Check if vote in progress
     *
     * @return boolean true/false
     */
    private function isVoteInProgress() {
        return $this->vote instanceof \MAM\Plugins\Voting\Vote;
    }

    /**
     * Cancels the function if next map loads
     *
     * @return false if this function is disabled
     */
    public function triggerNextMap() {
        $this->mapstart = time();
        if ($this->mod->getCV("voting", "ignoremapchange")) {
            return;
        }
        $this->vote = null;
    }

    /**
     * This function is called everytime
     *
     * @return void
     */
    public function triggerEveryTime() {
        if (!$this->isVoteInProgress()) {
            return;
        }

        if ($this->vote->selfcanceled) {
            $this->vote->cancel();
            $this->vote = null;
            $this->logging->info("Voting: Vote has been self canceled");
            return;
        }

        if (time() - $this->vote->getStarted() > $this->mod->getCV("voting", "duration")) {
            $this->logging->info("Voting: Vote ended (Yes: " . $this->vote->getYes() . ", No: " . $this->vote->getNo() . ")");
            $this->vote->end();
            if (array_key_exists($this->vote->getStarter(), $this->players)) {
                $this->players[$this->vote->getStarter()]->voting_votesstarted ++;
            }
            $this->vote = null;
            return;
        }

        $this->vote->triggerEveryTime();
    }

    /**
     * Command for !pass
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return  void
     */
    public function commandPass($guid, $parameters) {
        if (!$this->isVoteInProgress()) {
            return;
        }

        $this->vote->pass();
        $this->vote = null;
        $this->logging->info("Voting: Vote has been passed by admin");
    }

    /**
     * Command for !cancel
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return void
     */
    public function commandCancel($guid, $parameters) {
        if (!$this->isVoteInProgress()) {
            return;
        }

        $this->vote->cancel();
        $this->vote = null;
        $this->logging->info("Voting: Vote has been canceled by admin");
    }

    /**
     * Command for admin vote !avote
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return  void
     */
    public function commandAvote($guid, $parameters) {
        if (!$this->mod->getCV("voting", "enabled")) {
            return;
        }
        if ($this->isVoteInProgress()) {
            return;
        }

        $parameters = array_map("strtolower", $parameters);

        if (empty($parameters[0]) || !array_key_exists($parameters[0], $this->votes)) {
            $this->players[$guid]->say($this->mod->getLngString("allowedVotes", "<ALLOWED>", implode(", ", array_keys($this->votes))));
            return;
        }

        $vote = array_shift($parameters);

        try {
            $var = __NAMESPACE__ . '\\' . $this->votes[$vote];
            $this->vote = new $var($this, $guid, $parameters);
            $this->logging->info("Voting: Vote forced: $vote (Parameters: " . implode(", ", $parameters) . ")");
        } catch (\MAM\Plugins\Voting\VoteException $e) {
            $this->players[$guid]->say($e->getMessage());
        }
    }

    /**
     * Catch yes and no without !
     *
     * @param array $args
     * @return void
     */
    public function playerSay($args) {
        if (!$this->isVoteInProgress()) {
            return;
        }

        list($guid, $msg, $executed) = $args;
        if ($executed) {
            return;
        }

        $msg = strtolower(trim($this->mod->removecolor($msg)));

        if ($msg == "yes" || $msg == "ja") {
            $this->mod->executeCommand($this->mod->getCV("daemon", "cmdprefix") . "yes", $guid, $executed);
        } elseif ($msg == "no" || $msg == "nein") {
            $this->mod->executeCommand($this->mod->getCV("daemon", "cmdprefix") . "no", $guid, $executed);
        }
    }

    /**
     * Player left the server
     * @param string $guid Guid of Player
     */
    public function playerQuit($guid) {
        if ($this->isVoteInProgress()) {
            $this->vote->playerQuit($guid);
        }
    }
}
