<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\WelcomeMessages;

use MAM\Daemon\Core\Utils;

/**
 * Plugin Welcome Messages
 *
 * sends a welcome message to every player
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class WelcomeMessages extends Utils {

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        /* Define default Cvars */
        $this->mod->setDefaultCV('welcomemessages', 'enabled', 0);
        $this->mod->setDefaultCV('welcomemessages', 'time', 120);
        $this->mod->setDefaultCV('welcomemessages', 'whisper', 0);
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();
        if ($this->mod->getEngine() != 'frostbite3') {
            $this->mod->registerEvent('playerJoined', 'send', $this);
        } else {
            $this->mod->registerEvent('playerIPUpdated', 'send', $this);
        }
    }

    /**
     * Sends the welcome message
     *
     * @param stsring $guid
     * @return boolean false on fail
     */
    public function send($guid) {

        /* Show Messages only if plugin is enabled */
        if (!$this->mod->getCV('welcomemessages', 'enabled')) {
            return false;
        }

        /* Get the group of the player */
        $group = $this->players[$guid]->getGroup();

        /* Is there a welcome message for this group? */
        if (!$this->mod->existsCV('welcomemessages', $group)) {
            return false;
        }

        /* Search and Replace text patterns */
        $search = array(
            '<PLAYER_NAME>',
            '<GROUP_NAME>',
            '<COUNTRY_NAME>',
            '<COUNTRY_CODE>',
            '<PLAYER_NUMBER>',
            '<VISITS>',
            '<ONLINETIME>',
        );

        /* Get geoip data */
        try {
            $geoip_data = $this->geoip->getLocationData($this->players[$guid]->getIP());

            if (isset($geoip_data['country']['iso_code'])) {
                $country_code = $geoip_data['country']['iso_code'];
            } else {
                $country_code = 'xy';
            }

            if (isset($geoip_data['country']['names']) && $this->mod->getCV('daemon', 'language') == 'english') {
                $country_name = $geoip_data['country']['names']['en'];
            } else if (isset($geoip_data['country']['names']) && $this->mod->getCV('daemon', 'language') == 'german') {
                $country_name = $geoip_data['country']['names']['de'];
            } else {
                $country_name = 'cosmic space (using IPv9)';
            }
        } catch (\MAM\Daemon\Libraries\GeoIP\GeoIpException $e) {
            $this->logging->notice('Geoip: ' . $e);
            $country_name = 'cosmic space (using IPv9)';
            $country_code = 'xy';
        }

        $player_database_data = $this->players[$guid]->getPlayerInfos();


        $replace = array(
            $this->players[$guid]->getName(),
            $this->mod->getLongGroupName($group),
            $country_name,
            $country_code,
            $this->players[$guid]->getUserId(),
            $player_database_data->connections,
            \MAM\Daemon\Libraries\Helper\Helper::convertSeconds((int) $player_database_data->onlinetime, '%d <DAYS>, %h <HOURS> <AND> %i <MINUTES>'),
        );

        /* Send the message */
        if ($this->mod->getCV('welcomemessages', 'whisper')) {
            $this->players[$guid]->say(str_replace($search, $replace, $this->mod->getCV('welcomemessages', $group)));
        } else {
            $this->rcon->rconSay(str_replace($search, $replace, $this->mod->getCV('welcomemessages', $group)));
        }
    }
}
